
local G, W, H
local sz = 3

local seed = os.time()

local sin = math.sin
local r = math.random
local twopi = math.pi*2
local function oscillate (amplitude, frequency, phase)
   return amplitude * sin(frequency * twopi * time + phase)
end

local light_position = {{}, {}, {}, {}, {}}
local light_diffuse = {{2, 0.5, 0}, {1, 1, 0}, {1, 0, 1}, {0, 1, 1}, {0, 0, 2}}
local num_lights = 5
math.randomseed(seed)

function send_lights ()
   effect:send("light_position", unpack(light_position, 1, num_lights))
   effect:send("light_diffuse", unpack(light_diffuse, 1, num_lights))
end

function draw_lights ()
   G.setPixelEffect()
   G.setBlendMode("additive")
   for i = 1, num_lights do
      local diffuse = light_diffuse[i]
      local x = light_position[i][1]
      local y = H()/sz - light_position[i][2]
      local z = light_position[i][3]

      local r = diffuse[1]*255
      local g = diffuse[2]*255
      local b = diffuse[3]*255

      G.setColor(r, g, b)
      -- G.point(x, y)
      G.draw(globe, x, y-z, 0, 1, 1, 9, 9)
   end
   G.setBlendMode("alpha")
end

function love.load ()
   G = love.graphics
   W = G.getWidth
   H = G.getHeight
   G.setCaption "Multiple Lights Test"
   G.setBackgroundColor(35, 30, 65)
   G.setPoint(1, "rough")

   time = 0

   G.setDefaultImageFilter("nearest", "nearest")
   globe = G.newImage "globe.png"
   img = G.newImage "pithos.png"
   diffuse = G.newQuad(0, 0, 32, 43, 96, 43)

   img2 = G.newImage "chair.png"
   diffuse2 = G.newQuad(0, 0, 32, 50, 96, 50)

   canvas = G.newCanvas(W()/sz, H()/sz)
   canvas:setFilter("nearest", "nearest")

   effect = G.newPixelEffect "multilights.glsl"
end

function love.update (dt)
   time = time + dt
   z = 20 + oscillate(15, 0.1, 0)

   local x, y = love.mouse.getPosition()
   x = x/sz
   y = y/sz
   y = H()/sz - y

   light_position[1][1] = x
   light_position[1][2] = y
   light_position[1][3] = z
   -- effect:send("light_pos", {x, y, z})

   math.randomseed(seed)
   for i = 2, num_lights do
      light_position[i][1] = W()/sz/2 + oscillate(50, 41/400, r()*5)
      light_position[i][2] = H()/sz/2 + oscillate(50, 47/400, r()*5)
      light_position[i][3] = 20 + oscillate(15, 0.2, 0)      
   end

   send_lights()
end

function love.draw ()
   canvas:clear()
   G.setCanvas(canvas)
   draw()
   G.setCanvas()
   G.setPixelEffect()
   G.setColor(255, 255, 255)
   G.draw(canvas, 0, 0, 0, sz, sz)
end

function printf (...)
   G.print(string.format(...), 20, 10)
end

function draw ()
   G.setPixelEffect()
   G.setColor(220, 190, 0)
   G.setPixelEffect(effect)
   G.setColor(255, 255, 255)
   G.drawq(img, diffuse, W()/sz/2 - 34, H()/sz/2 - 18, 0, 1, 1, 16, 27)
   G.drawq(img2, diffuse2, W()/sz/2 + 34, H()/sz/2 + 18, 0, 1, 1, 16, 27)
   G.setPixelEffect()
   draw_lights()
end

